#ifndef FPUi2c
#define FPUi2c
#include "FPUdefs.bas"

'==============================================================================
' Copyright  Micromega Corporation 2002-2007. All rights reserved.
'
' @file		FPUi2c.bas
' @target:	ARMmite/ARMexpress
'
' This file contains support code for using the uM-FPU V3 floating point
' coprocessor with the ARMexpress or ARMmite over an I2C interface.
'
' @author Cam Thompson, Micromega Corporation
' @version
'	March 31, 2008
'	- updated for ARMbasic V7
'   August 13, 2007
'   - use #define for FpuCS definition
'	August 2, 2007
'	- original version
'==============================================================================

#include <I2C.bas>

'==============================================================================
'-------------------- uM-FPU V3.1 I2C definitions -----------------------------
'==============================================================================

const	FpuID		= $C8				' I2C address for FPU
const	FpuSDA		= 0					' I2C SDA pin
const	FpuSCL		= 1					' I2C SCL pin

'==============================================================================
'-------------------- uM-FPU V3.1 SPI support routines ------------------------
'==============================================================================

SUB Fpu_Wait
    while 1
		str$(0) = 0
		I2CIN(FpuSDA, FpuSCL, FpuID, 1, str$, 1, str$)
		if str$(0) = 0 then return
    loop
END SUB

SUB Fpu_Write(bval1)
	str$(0) = 0
	str$(1) = bval1
	I2COUT(FpuSDA, FpuSCL, FpuID, 2, str$)
END SUB

SUB Fpu_Write2(bval1, bval2)
	str$(0) = 0
	str$(1) = bval1
	str$(2) = bval2
	I2COUT(FpuSDA, FpuSCL, FpuID, 3, str$)
END SUB

SUB Fpu_Write3(bval1, bval2, bval3)
	str$(0) = 0
	str$(1) = bval1
	str$(2) = bval2
	str$(3) = bval3
	I2COUT(FpuSDA, FpuSCL, FpuID, 4, str$)

END SUB

SUB Fpu_Write4(bval1, bval2, bval3, bval4)
	str$(0) = 0
	str$(1) = bval1
	str$(2) = bval2
	str$(3) = bval3
	str$(4) = bval4
	I2COUT(FpuSDA, FpuSCL, FpuID, 5, str$)
END SUB

SUB Fpu_Write5(bval1, bval2, bval3, bval4, bval5)
	str$(0) = 0
	str$(1) = bval1
	str$(2) = bval2
	str$(3) = bval3
	str$(4) = bval4
	str$(5) = bval5
	I2COUT(FpuSDA, FpuSCL, FpuID, 6, str$)
END SUB

SUB Fpu_Write6(bval1, bval2, bval3, bval4, bval5, bval6)
	str$(0) = 0
	str$(1) = bval1
	str$(2) = bval2
	str$(3) = bval3
	str$(4) = bval4
	str$(5) = bval5
	str$(6) = bval6
	I2COUT(FpuSDA, FpuSCL, FpuID, 7, str$)
END SUB

SUB Fpu_Write7(bval1, bval2, bval3, bval4, bval5, bval6, bval7)
	str$(0) = 0
	str$(1) = bval1
	str$(2) = bval2
	str$(3) = bval3
	str$(4) = bval4
	str$(5) = bval5
	str$(6) = bval6
	str$(7) = bval7
	I2COUT(FpuSDA, FpuSCL, FpuID, 8, str$)
END SUB

SUB Fpu_Write8(bval1, bval2, bval3, bval4, bval5, bval6, bval7, bval8)
	str$(0) = 0
	str$(1) = bval1
	str$(2) = bval2
	str$(3) = bval3
	str$(4) = bval4
	str$(5) = bval5
	str$(6) = bval6
	str$(7) = bval7
	str$(8) = bval8
	I2COUT(FpuSDA, FpuSCL, FpuID, 9, str$)
END SUB

SUB Fpu_WriteWord(wval)
	str$(0) = 0
	str$(1) = wval>>8
	str$(2) = wval
	I2COUT(FpuSDA, FpuSCL, FpuID, 3, str$)
END SUB

SUB Fpu_WriteLong(lval)
	str$(0) = 0
	str$(1) = lval>>24
	str$(2) = lval>>16
	str$(3) = lval>>8
	str$(4) = lval
	I2COUT(FpuSDA, FpuSCL, FpuID, 5, str$)
END SUB

SUB Fpu_WriteFloat(fval)
	Fpu_WriteLong(fval)
END SUB

SUB Fpu_WriteString(BYREF s as string)
	str$ = "x" + s
	str$(0) = 0
	I2COUT(FpuSDA, FpuSCL, FpuID, len(s)+2, str$)
END SUB
	
SUB Fpu_ReadDelay
'	enough delay provided by call overheads
'	time1 = TIMER
'	while ((TIMER - time1) < 15)
'	loop
END SUB

FUNCTION Fpu_Read
	Fpu_ReadDelay
	str$(0) = 0
	I2CIN(FpuSDA, FpuSCL, FpuID, 1, str$, 1, str$)
	return str$(0)
END FUNCTION

FUNCTION Fpu_ReadWord
	Fpu_ReadDelay
	str$(0) = 0
	I2CIN(FpuSDA, FpuSCL, FpuID, 1, str$, 2, str$)
	return (str$(0)<<8) + str$(1)
END FUNCTION

FUNCTION Fpu_ReadLong
	Fpu_ReadDelay
	str$(0) = 0
	I2CIN(FpuSDA, FpuSCL, FpuID, 1, str$, 4, str$)
	return (str$(0)<<24) + (str$(1)<<16) + (str$(2)<<8) + str$(3)	
END FUNCTION

FUNCTION Fpu_ReadFloat
	return Fpu_ReadLong	
END FUNCTION

FUNCTION Fpu_ReadString(opcode) as string
	Fpu_Wait
	if opcode <> READSEL then opcode = READSTR
	Fpu_Write(opcode)
	gosub FPU_ReadDelay
	str$(0) = 0
	I2CIN(FpuSDA, FpuSCL, FpuID, 1, str$, 0, str$)
	return str$
END FUNCTION

FUNCTION Fpu_ReadStatus
	Fpu_Wait
	Fpu_Write(READSTATUS)
    return Fpu_Read
END FUNCTION

FUNCTION Fpu_Reset
	str$(0) = 1
	str$(1) = 0
	I2COUT(FpuSDA, FpuSCL, FpuID, 2, str$)
	wait(10)
	Fpu_Write(SYNC)
	return Fpu_Read
END FUNCTION
	
SUB Print_Version
	Fpu_Write(VERSION)
	print Fpu_ReadString(READSTR);
END SUB

SUB Print_Float(format)
	Fpu_Write2(FTOA, format)
	print Fpu_ReadString(READSTR);
END SUB

SUB Print_Long(format)
	Fpu_Write2(LTOA, format)
	print Fpu_ReadString(READSTR);
END SUB

SUB Print_FpuString(opcode)
	print Fpu_ReadString(opcode);
END SUB

#endif