'==============================================================================
' Copyright  Micromega Corporation 2002-2008. All rights reserved.
'
' @file: 	demo1.bas
' @target:	ARMmite/ARMexpress
'
' This program provides an example of using the uM-FPU V3 floating point
' coprocessor with the ARMmite/ARMexpress using an SPI interface. It takes
' temperature readings from a DS1620 digital thermometer, converts them to
' floating point and displays them in degrees Celsius and degrees Fahrenheit.
'
' @author Cam Thompson, Micromega Corporation
' @version
'	May 28, 2008
'	- corrected comments for DS1620 pins
'   March 31, 2008
'	- updated for ARMbasic V7
'	August 2, 2007
'	- updated for uM-FPU V3.1
'	November 9, 2006
'	- original version
'==============================================================================

'-------------------- uM-FPU V3.1 definitions ---------------------------------

#include "FPUspi.bas"					' select either SPI or I2C
'#include "FPUi2c.bas"

'-------------------- DS1620 pin definitions ----------------------------------

#define SHIFTclkNEGATIVE 1				
#include <shift.bas>

const	DS_DATA     = 10				' DS1620 data (1K to pin 1)
const	DS_CLK      = 11				' DS1620 clock (pin 2)
const	DS_RST		= 12				' DS1620 reset/enable (pin 3)

'-------------------- uM-FPU register definitions -----------------------------

const	DegC        = 8					' degrees Celsius
const	DegF        = 9					' degrees Fahrenheit
const	F1_8        = 10				' constant 1.8

'==============================================================================
'-------------------- subroutines ---------------------------------------------
'==============================================================================

'-------------------- Init_DS1620 ---------------------------------------------

SUB Init_DS1620
	IO(DS_RST) = 0						' initialize pin states
	IO(DS_CLK) = 1
	WAIT(100)

	OUT(DS_RST) = 1						' configure for CPU control
	shiftValues(0) = $0C
	shiftCounts(0) = 8
	shiftValues(1) = $02
	shiftCounts(1) = 8
	SHIFTOUT(DS_DATA, DS_CLK, 1, 2)
	OUT(DS_RST) = 0
	WAIT(100)

	OUT(DS_RST) = 1						' start temperature conversions
	shiftValues(0) = $EE
	shiftCounts(0) = 8
	SHIFTOUT(DS_DATA, DS_CLK, 1, 1)
	OUT(DS_RST) = 0
	WAIT(1000)							' wait for first conversion
END SUB

'-------------------- Read_DS1620 ---------------------------------------------

FUNCTION Read_DS1620
	dim n as integer
	OUT(DS_RST) = 1						' read temperature value
	shiftValues(0) = $AA
	shiftCounts(0) = 8
	SHIFTOUT(DS_DATA, DS_CLK, 1, 1)

	shiftCounts(0) = 9	
	SHIFTIN(DS_DATA, DS_CLK, 1, 1)
	n = shiftValues(0)
	OUT(DS_RST) = 0						' extend the sign bit
	if (n AND 0x100) then n = n OR 0xFFFFFF00
	return n
END FUNCTION

'==============================================================================
'-------------------- main routine --------------------------------------------
'==============================================================================

Main:
	print
	print "Demo 1"
	print "------"

	if Fpu_Reset <> SYNC_CHAR then		' reset FPU and check synchronization
		print "uM-FPU not detected"
		end
	else
		Print_Version					' display FPU version number
		print
	endif

	Init_DS1620							' initialize DS1620

	Fpu_Write3(SELECTA, F1_8, ATOF)		' load floating point constant 1.8
	Fpu_WriteString("1.8")
	Fpu_Write(FSET0)

'-------------------- main loop -----------------------------------------------

	do
		rawTemp = read_DS1620			' get temperature reading from DS1620
		print "Raw Temp:   $"; HEX(rawTemp)
										' send rawTemp to uM-FPU
										' convert to floating point
										' store in register
										' divide by 2 to get degrees Celsius
		Fpu_Write3(SELECTA, DegC, LOADWORD)
		Fpu_WriteWord(rawTemp)
		Fpu_Write3(FSET0, FDIVI, 2)

										' degF = degC * 1.8 + 32
		Fpu_Write4(SELECTA, DegF, FSET, DegC)
		Fpu_Write4(FMUL, F1_8, FADDI, 32)

		print "Degrees C: ";			' display degrees Celsius
		Fpu_Write2(SELECTA, DegC)
		Print_Float(51)
		print

		print "Degrees F: ";			' display degrees Fahrenheit
		Fpu_Write2(SELECTA, DegF)
		Print_Float(51)
		print

		WAIT(2000)						' delay 2 seconds
		print
	loop
