'==============================================================================
' Copyright  Micromega Corporation 2002-2008. All rights reserved.
'
' @file:	graph.bas
' @target:	ARMmite/ARMexpress
'
' This program provides an example of using the uM-FPU V3 floating point
' coprocessor with the ARMmite/ARMexpress using an SPI interface. It draws a
' simple graph of the sine and cosine functions in the terminal window.
'
' @author Cam Thompson, Micromega Corporation
' @version
'   March 30, 2008
'	- updated for ARMbasic V7
'	August 2, 2007
'	- updated for uM-FPU V3.1
'	November 9, 2006
'	- original version
'==============================================================================

'-------------------- uM-FPU V3.1 definitions ---------------------------------

#include "FPUspi.bas"					' select either SPI or I2C
'#include "FPUi2c.bas"

'-------------------- uM-FPU register definitions -----------------------------

const	Angle		= 1					' angle
const	Point		= 2					' point on graph

'-------------------- constants -----------------------------------------------

const	Space		= 32				' ASCII character values
const	Asterisk	= 42
const	VerticalLine= 124

'==============================================================================
'-------------------- subroutines ---------------------------------------------
'==============================================================================

SUB Graph
	dim p as integer
	dim x as integer
	dim tmp as integer
	
	' Point = Fix(Round(Angle * 10))
	'-------------------------------
	Fpu_Wait
	Fpu_Write4(SELECTA, Point, FSET, Angle)
	Fpu_Write3(FMULI, 10, FIXR)

	'Read the lower 8 bits of long value
	'-----------------------------------
	Fpu_Wait
	Fpu_Write(LREADBYTE)
	p = Fpu_Read
	if (p AND $80) then p = p OR $FFFFFF00

	' add offset to midpoint of graph
	'--------------------------------
	p = p + 10

	'plot the point on graph
	'-----------------------
	for x = 0 TO 20
		if (x = p) then
			tmp = Asterisk
		elseIF (x = 10) then
			tmp = VerticalLine
		else
			tmp = Space
		endif
		print CHR(tmp);
	next

	'display the original angle in 8.5 format
	'----------------------------------------
	print " ";
	Fpu_Write2(SELECTA, Angle)
	Print_Float(85)
	print
END SUB

'==============================================================================
'-------------------- main routine --------------------------------------------
'==============================================================================

Main:
	print
	print "Graph demo"
	
	if Fpu_Reset <> SYNC_CHAR then		' reset FPU and check synchronization
		print "uM-FPU not detected"
		end
	else
		Print_Version					' display FPU version number
		print
	endif

	' graph sine from 0 TO 360 degrees (2*PI radians)
	'------------------------------------------------
	print
	print "Graph of sine"
	print "-1        0         1"
	print "---------------------"

	for cnt = 0 TO 20
		' Angle = sin(pi * 2 * cnt / 20)
		'-------------------------------
		Fpu_Wait
		Fpu_Write4(SELECTA, Angle, LOADPI, FSET0)
		Fpu_Write4(FMULI, cnt*2, FDIVI, 20)
		Fpu_Write(F_SIN)
		Graph						' graph the result
	next

	' graph cosine from 0 TO 360 degrees (2*PI radians)
	'--------------------------------------------------
	print
	print "Graph of cosine"
	print "-1        0         1"
	print "---------------------"
	
	for cnt = 0 TO 20
		' Angle = cos(pi * 2 * cnt / 20)
		'-------------------------------
		Fpu_Wait
		Fpu_Write4(SELECTA, Angle, LOADPI, FSET0)
		Fpu_Write4(FMULI, cnt*2, FDIVI, 20)
		Fpu_Write(F_COS)
		Graph						' graph the result
	next

Done:
	print "Done."
	end

