'==============================================================================
' Copyright  Micromega Corporation 2002-2008. All rights reserved.
'
' @file:	sample.bas
' @target:	ARMmite/ARMexpress
'
' This program provides an example of using the uM-FPU V3 floating point
' coprocessor with the ARMmite/ARMexpress using an SPI interface. This demo
' takes an integer value representing the diameter of a circle in centimeters,
' converts the value to inches, then calculates the circumference and area in
' inches and square inches.
'
' @author Cam Thompson, Micromega Corporation
' @version
'   March 30, 2008
'	- updated for ARMbasic V7
'	August 2, 2007
'	- updated for uM-FPU V3.1
'	November 9, 2006
'	- original version
'==============================================================================

'-------------------- uM-FPU V3.1 definitions ---------------------------------

#include "FPUspi.bas"					' select either SPI or I2C
'#include "FPUi2c.bas"

'-------------------- uM-FPU register definitions -----------------------------

const	DiameterIn		= 1				' dimeter in inches
const	Circumference	= 2				' circumference
const	Area			= 3				' area
const	Pi				= 4				' constant pi

'==============================================================================
'-------------------- main routine --------------------------------------------
'==============================================================================

Main:
	print
	print "Conversion Example"
	print "------------------"

	' Reset the uM-FPU and print version.
	'------------------------------------
	if Fpu_Reset <> SYNC_CHAR then		' reset FPU and check synchronization
		print "uM-FPU not detected"
		end
	else
		Print_Version					' display FPU version number
		print
	endif

	' Load constant for later use.
	'-----------------------------
	Fpu_Write4(SELECTA, Pi, LOADPI, FSET0)

	' Get diameter in centimeters.  The value would typically come from a
	' sensor reading, but in this example an assumed value of 25 is used.
	'--------------------------------------------------------------------
	diameterCm = 25
	print "Diameter (cm):       "; diameterCm
	
	' convert inches to centimeters
	'------------------------------
	Fpu_Write6(SELECTA, DiameterIn, FSETI, diameterCm, FCNV, 5)
	print "Diameter (in.):      ";
	Print_Float(0)
	print
	
	' circumference = diameter * pi
	'------------------------------
	Fpu_Write6(SELECTA, Circumference, FSET, DiameterIn, FMUL, Pi)
	print "Circumference (in.): ";
	Print_Float(0)
	print
	
	' area = (diameter / 2)^2 * pi
 	'-----------------------------
	Fpu_Write4(SELECTA, Area, FSET, DiameterIn)
	Fpu_Write6(FDIVI, 2, FMUL, Area, FMUL, Pi)
	print "Area (sq.in.):       ";
	Print_Float(0)
	print
	
	print "Done."
	end
